/*============================================================================*

	SUITE:		e-Scrabble
	FILE:		user_events.c
	
	COPYRIGHT NOTICE:
	
	Copyright (C) 2001 Players Inc, All Rights Reserved

	THIS MATERIAL IS PROVIDED FOR ACADEMIC USE ONLY, you may modify it
	provided this copyright notice is preserved, however you may not
	redistribute it, or use it for commercial purposes without prior
	written permission of the authors.

	Direct questions, comments, or bug reports to:
	aul@ecf.utoronto.ca
	ticku@ecf.utoronto.ca
	varia@ecf.utoronto.ca	

*============================================================================*/


#include <stdio.h>
#include "scrabble.h"
#include "user_events.h"
#include "pci_api.h"
#include "utils.h"
#include "draw_utils.h"


extern Bmove board_move;
extern Location_Status **board_tile_state;
extern Coord board_dim;
extern gboolean challenge_status;
extern gboolean make_move_status;
extern char* status_messages[];
extern int rack_size;
extern Tile *rack_state;

static GtkWidget *dialog;


static void get_board_move ();
static void setup_new_game_gui ();
static void clicked_help (GtkWidget *, gpointer);

static void user_start_game (GtkWidget *, gpointer);
static void user_cancel_game (GtkWidget *, gpointer);
static void user_invite_player (GtkWidget *, gpointer);
static void invitee_picked ( GtkWidget *, gint, gint,
				GdkEventButton *, gpointer);
static void invitee_unpicked ( GtkWidget *, gint, gint,
				GdkEventButton *, gpointer);

static GtkWidget *new_dialog;
static int invited_player = -1;
static PlayerIDList pidlist;


/*=================== MENU COMMAND USER EVENTS ==================*/

/* procedure menu_game_new(widget: GtkWidget *, data: gpointer) returns void
 * effects: requests new game from GM and shows available player list
 * modifies: opens new game dialog box on screen */

void
menu_game_new (GtkWidget *widget, gpointer data)
{
	int retVal;
	pidlist.players = NULL;

	/* temporary fix */
	user_start_game(NULL, NULL);
	return;
	/* end temporary fix */
	
	
	retVal = pci_new_game ( TRUE, &pidlist );
		
	if (retVal == SUCCESS)
	  {
		setup_new_game_gui ();
	  }
	else
	  {
		print_msg( status_messages[retVal] );
		s_free( pidlist.players );
	  }
	
}


/* procedure menu_game_resign(widget: GtkWidget *, data: gpointer returns void)
 * effects: sends cancel/resign game request to GM */

void
menu_game_resign (GtkWidget *widget, gpointer data)
{
	int retVal = pci_cancel_game();
	
	if( retVal != SUCCESS )
		print_msg( status_messages[retVal] );			  		
	else {
		int i,j;
		for(i=0; i<rack_size; i++ ) {
			rack_state[i] = TILE_EMPTY;
			clear_tile(TRUE, i, j);
		}
		for(i=0; i<board_dim.x; i++ )
			for(j=0; j<board_dim.y; j++) {
				Location_Status *loc = &(board_tile_state[i][j]);
			 	if( !(loc->fixed) && loc->t != TILE_EMPTY) {
					loc->t = TILE_EMPTY;
					clear_tile(FALSE, i, j);
				}
			}
		print_msg( "> Resignation was Successful\n" );
	}
}


/* procedure menu_game_save(widget: GtkWidget *, data: gpointer) returns void
 * effects: sends rack to GM for save game request */

void
menu_game_save (GtkWidget *widget, gpointer data)
{
 int retVal;

 retVal = pci_save_game();

 if(retVal == SUCCESS)
  {
   print_msg("> Save game was successfull\n");

  }
 else
   print_msg( status_messages[retVal] );	 
}

void
menu_add_ap( GtkWidget *widget, gpointer data)
{
	int retVal;
	retVal = pci_create_AP();
	if( retVal == SUCCESS )
	{
		print_msg("> AP Request sent successfully\n");
	}
	else
		print_msg( status_messages[retVal] );


}

/* procedure menu_game_restore(widget: GtkWidget *, data: gpointer) returns void
 * effects: sends restore game request to GM and shows available player list
 * modifies: opens new game dialog box on screen */

void
menu_game_restore (GtkWidget *widget, gpointer data) 
 {
  int retVal;
  Scores s;

  retVal = pci_restore_game(&s);
  s.players = NULL;
  s.scores = NULL;



  if (retVal == SUCCESS)
   {
    s_free(s.players);
    s_free(s.scores);
    print_msg("> Restore game was successfull\n");
   }
  else
    print_msg( status_messages[retVal] );	 


}


/* procedure menu_game_end(widget: GtkWidget *, data: gpointer) returns void
 * effects: sends cancel game request to GM and exits program
 * modifies: exits program */

void
menu_game_end(GtkWidget *widget, gpointer data)
{
	pci_cancel_game ();
	gtk_main_quit();
}


void
menu_help_about (GtkWidget *widget, gpointer data)
{
	GtkWidget *okay_button;
	GtkWidget *help_text;
	GtkWidget *vscroll;
	GtkWidget *hbox;
	FILE *in;
	char temp[2];

	dialog = gtk_dialog_new ();	
	gtk_window_set_policy (GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
	gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
	gtk_window_set_title (GTK_WINDOW(dialog), "eScrabble Help");
	
	gtk_widget_set_usize (dialog, 500, 500);
	
	okay_button = gtk_button_new_with_label("Okay");


	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->action_area),
								okay_button);
	hbox = gtk_hbox_new (FALSE, 0);	
	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), hbox);							

	help_text = gtk_text_new (NULL, NULL);
	gtk_text_set_editable ( GTK_TEXT (help_text), FALSE);
	gtk_text_set_word_wrap ( GTK_TEXT (help_text), TRUE);


	gtk_text_set_point (GTK_TEXT(help_text), 0);
	in = fopen("help.txt", "r");
	temp[1] = '\0';
	while(fscanf(in, "%c", temp) != EOF)
	 {
	  gtk_text_insert(GTK_TEXT(help_text), NULL, NULL, NULL, temp, -1);
	 }
	
	fclose(in);

	gtk_widget_show (hbox);

	gtk_box_pack_start (GTK_BOX (hbox), help_text, TRUE, TRUE, FALSE);
	gtk_widget_show (help_text);
	
	vscroll = gtk_vscrollbar_new (GTK_TEXT (help_text)->vadj);
	gtk_box_pack_end (GTK_BOX (hbox), vscroll, FALSE, 0, 0);
	gtk_widget_show (vscroll);
									
								

	gtk_signal_connect (GTK_OBJECT (okay_button), "clicked",
						GTK_SIGNAL_FUNC (clicked_help), NULL);

	
	gtk_widget_show_all (dialog);

	
}

void
clicked_help (GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(dialog));
}


/*================== BUTTON COMMAND USER EVENTS =================*/


void
button_try_move (GtkWidget *widget, gpointer data)
{
	if (board_move.is_swap)
	  {
		print_msg("> Cannot Try Move:  Currently swapping tiles\n");
	  }
	else
	  {
		int retVal;
		int s;
		
		board_move.tiles_in_move = 0;
		get_board_move ();
		retVal = pci_try_move (board_move, &s);
			
		if( retVal == SUCCESS )
		  {
			char score[5];
			print_msg( "> TryMove returned score of: " );
			sprintf(score, "%i\n", s);
			print_msg( score );
		  }
		else
		  {
			print_msg( status_messages[retVal] );			  
		  }
	  }
}


void
button_pass (GtkWidget *widget, gpointer data)
{
	if (board_move.is_swap)
	  {
	  	int retVal;
		Scores s;
		s.players = NULL;
		s.scores = NULL;
		
		gtk_label_set_text(GTK_LABEL(GTK_BIN(widget)->child), "Swap Tiles");
		retVal = pci_make_move (board_move, &s);
		board_move.is_swap = FALSE;

		if( retVal == SUCCESS )
		  {
			print_msg( "> Swap was made successfully\n" );
			make_move_status = FALSE;
		  }
		else
		  {
			print_msg( status_messages[retVal] );
			refresh_rack(board_move.t, board_move.tiles_in_move);
		  }
		
		print_scores (&s);
		s_free(s.players);
		s_free(s.scores);
	  }
	else
	  {
		if( make_move_status )
		  {
			gtk_label_set_text(GTK_LABEL(GTK_BIN(widget)->child),
						"Click to Complete Swap");
		  	board_move.is_swap = TRUE;
			board_move.tiles_in_move = 0;
		  }
		else
		  {
			print_msg( "> Cannot Swap Tiles:  It is not your turn\n" );		  
		  }
	  }
	
}


void
button_make_move (GtkWidget *widget, gpointer data)
{
	if (make_move_status)
	  {
		if (board_move.is_swap)
		  {
			print_msg("> Cannot Make Move:  Currently swapping tiles\n");
		  }
		else
		  {
		  	int retVal;
			Scores s;
			s.players = NULL;
			s.scores = NULL;
			
			board_move.tiles_in_move = 0;
			get_board_move ();
			retVal = pci_make_move (board_move, &s);
			
			if( retVal == SUCCESS )
			  {
				print_msg( "> Move was made successfully\n" );
				make_move_status = FALSE;
				draw_my_move ();
				print_scores(&s);
			  }
			else
			  {
				print_msg( status_messages[retVal] );			  
			  }
			
			s_free(s.scores);
			s_free(s.players);
		  }
	  }
	else
	  {
		print_msg( "> Cannot Make Move:  It is not your turn\n" );	  
	  }
}


void
button_challenge (GtkWidget *widget, gpointer data)
{
	if (challenge_status)
	  {
	  	int retVal;
		challenge_status = FALSE;
		retVal = pci_challenge_move ();
		if( retVal == SUCCESS )
			print_msg( "> Challenge was successful\n");
		else
			print_msg( status_messages[retVal] );
	  }
	else
	  {
		print_msg( "> You have not been invited to challenge\n" );
	  }	
}



/*========================== UTILITY COMMANDS ========================*/

static void
get_board_move ()
{
	int i, j;
	
	for (i=0; i<board_dim.x; i++)
	  {
		for (j=0; j<board_dim.y; j++)
		  {
			if (board_tile_state[i][j].t != TILE_EMPTY &&
					!board_tile_state[i][j].fixed)
			  {
			  	board_move.t[board_move.tiles_in_move] = board_tile_state[i][j].t;
				board_move.pos[board_move.tiles_in_move].x = i;
				board_move.pos[board_move.tiles_in_move++].y = j;
			  }
		  }
	  }
}


/*======================== GUI FOR NEW GAME DIALOG =====================*/

static void
setup_new_game_gui ()
{
	int i;

	GtkWidget *invite;
	GtkWidget *cancel;
	GtkWidget *start;
	GtkWidget *hbox;
	GtkWidget *clist;
	GtkWidget *scroll;

	gchar *titles[1] = {"Player List"};
	gchar *pl_id[1] = {"Automated Player"};

	new_dialog = gtk_dialog_new ();	
	gtk_window_set_policy (GTK_WINDOW(new_dialog), FALSE, FALSE, TRUE);
	gtk_window_set_modal (GTK_WINDOW(new_dialog), TRUE);
	gtk_window_set_title (GTK_WINDOW(new_dialog), "New Game Request");
	gtk_widget_set_usize (GTK_WIDGET(new_dialog), 400, 150);

	invite = gtk_button_new_with_label("Invite Player");
	cancel = gtk_button_new_with_label("Cancel Game");
	start = gtk_button_new_with_label("Start Game");

	hbox = gtk_hbox_new (FALSE, 0);	
	gtk_box_pack_start (GTK_BOX (hbox), invite, TRUE, TRUE, FALSE);
	gtk_box_pack_start (GTK_BOX (hbox), start, TRUE, TRUE, FALSE);
	gtk_box_pack_start (GTK_BOX (hbox), cancel, TRUE, TRUE, FALSE);

	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scroll),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	
	clist = gtk_clist_new_with_titles (1, titles);
	gtk_clist_set_shadow_type (GTK_CLIST(clist), GTK_SHADOW_OUT);
	gtk_clist_set_column_width (GTK_CLIST(clist), 0, 150);

	gtk_clist_append( GTK_CLIST(clist), pl_id);
	for (i=0; i<pidlist.num_players; i++)
	  {
	  	char temp[10];
	  	sprintf(temp, "%i", pidlist.players[i]);
		pl_id[0] = temp;
	  	gtk_clist_append( GTK_CLIST(clist), pl_id );
	  }

	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(new_dialog)->vbox),
								scroll);					
	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(new_dialog)->action_area),
								hbox);
	gtk_container_add (GTK_CONTAINER(scroll), clist);	

	gtk_signal_connect (GTK_OBJECT (invite), "clicked",
						GTK_SIGNAL_FUNC (user_invite_player), NULL);

	gtk_signal_connect (GTK_OBJECT (cancel), "clicked",
						GTK_SIGNAL_FUNC (user_cancel_game), NULL);

	gtk_signal_connect (GTK_OBJECT (start), "clicked",
						GTK_SIGNAL_FUNC (user_start_game), NULL);

	gtk_signal_connect (GTK_OBJECT (clist), "select_row",
						GTK_SIGNAL_FUNC (invitee_picked), NULL);

	gtk_signal_connect (GTK_OBJECT (clist), "unselect_row",
						GTK_SIGNAL_FUNC (invitee_unpicked), NULL);

	
	gtk_widget_show_all (new_dialog);
}


static void
invitee_picked ( GtkWidget *clist, gint row, gint column,
					GdkEventButton *event, gpointer data)
{
	invited_player = row;
}

static void
invitee_unpicked ( GtkWidget *clist, gint row, gint column,
					GdkEventButton *event, gpointer data)
{
	invited_player = -1;
}


static void
user_start_game (GtkWidget *widget, gpointer data)
{
	int retVal;
	Scores s;
	s.players = NULL;
	s.scores = NULL;
	
/*	gtk_widget_destroy(GTK_WIDGET(new_dialog)); */
	retVal = pci_start_game( &s );
	
	if( retVal == SUCCESS )
	  {
		print_scores( &s );	  
	  }
	else
	  {
		print_msg( status_messages[retVal] );	  
	  }

	s_free( s.players );
	s_free( s.scores );	
	s_free( pidlist.players );
	
}



static void
user_cancel_game (GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(new_dialog));
	s_free( pidlist.players );
	pci_cancel_game ();
}



static void
user_invite_player (GtkWidget *widget, gpointer data)
{
	PlayerID pid;
	int retVal;

	if ( invited_player == -1 )
	  {
	  	print_msg( "> No Player Selected\n" );
		return;
	  }
	  
	if ( invited_player == 0 )
	  {
		pid = AUTO_PLAYER;
	  }
	else
	  {
	  	pid = pidlist.players[invited_player-1];
	  }
	
	retVal = pci_invite_player(pid);
	if (retVal == SUCCESS)
		print_msg("> Player Invited Successfully\n");
	else
		print_msg( status_messages[retVal] );
}
